/*
 *  Copyright (C) 2007  Thadeu Lima de Souza Cascardo <cascardo@holoscopio.com>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License along
 *  with this program; if not, write to the Free Software Foundation, Inc.,
 *  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 */


#include <atompub/atom.h>

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>

void
cgi_serve_request (AtomCtx *ctx)
{
  char *method = getenv ("REQUEST_METHOD");
  char *path = getenv ("PATH_INFO");
  if (method == NULL)
    return;
  if (path == NULL)
    {
      path = atom_config_get_str (ctx, "cgi", "index");
    }
  if (!strcmp (method, "GET"))
    {
      IRI *iri = iri_new ();
      AtomEntry *atom;
      AtomError *error;
      iri_set_path (iri, path);
      atom = atom_retrieve_resource (ctx, iri);
      iri_delete (iri);
      if (atom)
	{
	  char * str;
	  size_t len;
	  char *header = "Content-type: application/atom+xml\n\n";
	  write (1, header, strlen (header));
	  atom_entry_string (atom, &str, &len);
	  write (1, str, len);
	  g_free (str);
	  atom_entry_delete (atom);
	}
      else if ((error = atom_error_get (ctx)) != NULL)
	{
	  int code = atom_error_code (error);
	  char *message = atom_error_message (error);
	  fprintf (stdout, "Status: %d %s\n\n%s\n", code, message, message);
	}
      else
        {
	  fprintf (stdout, "Status: 500 Server error\n\nServer error\n");
	}
    }
  else
    {
      fprintf (stdout, "Status: 501 Not Implemented\n\n");
    }
}
