/*
 *  Copyright (C) 2007  Thadeu Lima de Souza Cascardo <cascardo@holoscopio.com>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License along
 *  with this program; if not, write to the Free Software Foundation, Inc.,
 *  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 */


#include <atompub/atom.h>
#include <glib.h>

struct _atom_backend
{
  AtomEntry * (*retrieve_resource) (AtomCtx *, IRI *);
  void (*enumerate_entries) (AtomCtx *, AtomEntry ***, size_t *);
};

AtomBackend *
atom_backend_new ()
{
  AtomBackend *backend;
  backend = g_slice_new (AtomBackend);
  backend->retrieve_resource = NULL;
  backend->enumerate_entries = NULL;
  return backend;
}

void
atom_backend_delete (AtomBackend *backend)
{
  g_slice_free (AtomBackend, backend);
}

void
atom_backend_retrieve_resource_set (AtomBackend *backend,
				    AtomEntry *retrieve_resource (AtomCtx *,
								  IRI *))
{
  backend->retrieve_resource = retrieve_resource;
}

void
atom_backend_enumerate_entries_set (AtomBackend *backend,
				    void enumerate_entries (AtomCtx *,
							    AtomEntry ***, size_t*))
{
  backend->enumerate_entries = enumerate_entries;
}

AtomEntry *
atom_retrieve_resource (AtomCtx *ctx, IRI *iri)
{
  AtomBackend *backend;
  backend = atom_backend (ctx);
  if (backend && backend->retrieve_resource)
    return backend->retrieve_resource (ctx, iri);
  return NULL;
}

void
atom_backend_enumerate_entries (AtomCtx *ctx, AtomEntry *** entries, size_t *len)
{
  AtomBackend *backend;
  backend = atom_backend (ctx);
  if (backend && backend->enumerate_entries)
    {
      backend->enumerate_entries (ctx, entries, len);
      return;
    }
}
